enum PinOptions {
    //% block="0"
    PIN_0,
    //% block="1"
    PIN_1,
    //% block="2"
    PIN_2,
    //% block="3"
    PIN_3,
    //% block="4"
    PIN_4,
    //% block="5"
    PIN_5,
    //% block="6"
    PIN_6,
    //% block="7"
    PIN_7,
    //% block="8"
    PIN_8,
    //% block="9"
    PIN_9,
    //% block="10"
    PIN_10,
    //% block="11"
    PIN_11,
    //% block="12"
    PIN_12,
    //% block="13"
    PIN_13,
    //% block="A0"
    PIN_A0,
    //% block="A1"
    PIN_A1,
    //% block="A2"
    PIN_A2,
    //% block="A3"
    PIN_A3,
    //% block="A4"
    PIN_A4,
    //% block="A5"
    PIN_A5
}

//% color="#AA278D" iconWidth=50 iconHeight=40
namespace U8G2_SSD1306_128X64 {
    let displayCommands: { x: string, y: string, text: string }[] = [];  // 存储所有显示命令的数组
    let hasPage1Function = false;  // 标记是否已经生成了page1函数

    //% block="初始化OLED显示屏 SCL [SCL_PIN] SDA [SDA_PIN]"
    //% blockType="command"
    //% SCL_PIN.shadow="dropdown" SCL_PIN.options="PinOptions" SCL_PIN.defl="PinOptions.PIN_A5"
    //% SDA_PIN.shadow="dropdown" SDA_PIN.options="PinOptions" SDA_PIN.defl="PinOptions.PIN_A4"
    export function initOLED(parameter: any, block: any) {
        let sclPin = parameter.SCL_PIN.code;
        let sdaPin = parameter.SDA_PIN.code;

        // 添加包含指令
        Generator.addInclude('Wire', '#include <Wire.h>');
        Generator.addInclude('U8g2lib', '#include <U8g2lib.h>');

        // 判断是否使用默认的硬件I2C引脚
        const isDefaultPins = (sclPin === 'PIN_A5' && sdaPin === 'PIN_A4');
        
        // 构建添加对象的代码
        let objectCode;
        if (isDefaultPins) {
            objectCode = ' u8g2(U8G2_R0, U8X8_PIN_NONE);';
            Generator.addObject(
                'u8g2',
                'U8G2_SSD1306_128X64_NONAME_1_HW_I2C',
                objectCode
            );
        } else {
            // 从引脚代码中提取数字
            let sclPinNum = sclPin.replace('PIN_', '');
            let sdaPinNum = sdaPin.replace('PIN_', '');
            objectCode = ` u8g2(U8G2_R0, ${sclPinNum}, ${sdaPinNum}, U8X8_PIN_NONE);`;
            Generator.addObject(
                'u8g2',
                'U8G2_SSD1306_128X64_NONAME_1_SW_I2C',
                objectCode
            );
        }

        // 生成初始化代码
        Generator.addSetup('u8g2_init', `  u8g2.setI2CAddress(0x3C*2);
  u8g2.begin();`);
        
        // 清空显示命令数组
        displayCommands = [];
        hasPage1Function = false;
    }

    //% block="显示屏page1显示文本行1: X [X1] Y [Y1] 文本 [TEXT1] 行2: X [X2] Y [Y2] 文本 [TEXT2] 行3: X [X3] Y [Y3] 文本 [TEXT3] 行4: X [X4] Y [Y4] 文本 [TEXT4]"
    //% blockType="command"
    //% X1.shadow="range" X1.params.min=0 X1.params.max=127 X1.defl=0
    //% Y1.shadow="range" Y1.params.min=0 Y1.params.max=63 Y1.defl=15
    //% TEXT1.shadow="string" TEXT1.defl="Line 1"
    //% X2.shadow="range" X2.params.min=0 X2.params.max=127 X2.defl=0
    //% Y2.shadow="range" Y2.params.min=0 Y2.params.max=63 Y2.defl=30
    //% TEXT2.shadow="string" TEXT2.defl="Line 2"
    //% X3.shadow="range" X3.params.min=0 X3.params.max=127 X3.defl=0
    //% Y3.shadow="range" Y3.params.min=0 Y3.params.max=63 Y3.defl=45
    //% TEXT3.shadow="string" TEXT3.defl="Line 3"
    //% X4.shadow="range" X4.params.min=0 X4.params.max=127 X4.defl=0
    //% Y4.shadow="range" Y4.params.min=0 Y4.params.max=63 Y4.defl=60
    //% TEXT4.shadow="string" TEXT4.defl="Line 4"
    export function generatePageFunction(parameter: any, block: any) {
        let x1 = parameter.X1.code;
        let y1 = parameter.Y1.code;
        let text1 = parameter.TEXT1.code;
        let x2 = parameter.X2.code;
        let y2 = parameter.Y2.code;
        let text2 = parameter.TEXT2.code;
        let x3 = parameter.X3.code;
        let y3 = parameter.Y3.code;
        let text3 = parameter.TEXT3.code;
        let x4 = parameter.X4.code;
        let y4 = parameter.Y4.code;
        let text4 = parameter.TEXT4.code;

        // 生成 page1 函数
        let page1Function = `void page1() {
  u8g2.setFont(u8g2_font_timR08_tf);
  u8g2.setFontPosTop();
  u8g2.setCursor(${x1}, ${y1});
  u8g2.print(${text1});
  u8g2.setCursor(${x2}, ${y2});
  u8g2.print(${text2});
  u8g2.setCursor(${x3}, ${y3});
  u8g2.print(${text3});
  u8g2.setCursor(${x4}, ${y4});
  u8g2.print(${text4});
}`;

        // 使用 Generator.addObject 确保 page1 函数是独立的
        Generator.addObject('page1', ' ', page1Function);
        
        // 添加 UTF8Print 设置到 setup
        Generator.addSetup('u8g2_utf8', 'u8g2.enableUTF8Print();');
    }

    //% block="执行显示page1"
    //% blockType="command"
    export function drawPage() {
        Generator.addCode(`
  u8g2.firstPage();
  do
  {
    page1();
  }while(u8g2.nextPage());
`);
    }
}
